@@ -92,7 +92,7 @@ If nil, only read articles will be expired."
 (defvar gnus-agent-spam-hashtb nil)
 (defvar gnus-agent-file-name nil)
 (defvar gnus-agent-send-mail-function nil)
-(defvar gnus-agent-article-file-coding-system 'no-conversion)
+(defvar gnus-agent-file-coding-system 'no-conversion)
 
 ;; Dynamic variables
 (defvar gnus-headers)
@@ -517,7 +517,7 @@ the actual number of articles toggled is returned."
     (let* ((gnus-command-method method)
 	   (file (gnus-agent-lib-file "active")))
       (gnus-make-directory (file-name-directory file))
-      (let ((coding-system-for-write gnus-agent-article-file-coding-system))
+      (let ((coding-system-for-write gnus-agent-file-coding-system))
 	(write-region (point-min) (point-max) file nil 'silent))
       (when (file-exists-p (gnus-agent-lib-file "groups"))
 	(delete-file (gnus-agent-lib-file "groups"))))))
@@ -526,9 +526,10 @@ the actual number of articles toggled is returned."
   (let* ((gnus-command-method method)
 	 (file (gnus-agent-lib-file "groups")))
     (gnus-make-directory (file-name-directory file))
-    (write-region (point-min) (point-max) file nil 'silent))
+    (let ((coding-system-for-write gnus-agent-file-coding-system))
+      (write-region (point-min) (point-max) file nil 'silent))
     (when (file-exists-p (gnus-agent-lib-file "active"))
-      (delete-file (gnus-agent-lib-file "active"))))
+      (delete-file (gnus-agent-lib-file "active")))))
 
 (defun gnus-agent-save-group-info (method group active)
   (when (gnus-agent-method-p method)
@@ -547,7 +548,7 @@ the actual number of articles toggled is returned."
 		     (concat "^" (regexp-quote group) " ") nil t)
 		(gnus-delete-line))
 	      (insert group " " (number-to-string (cdr active)) " "
-		      (number-to-string (car active)) "\n"))
+		      (number-to-string (car active)) " y\n"))
 	  (when (re-search-forward (concat (regexp-quote group) " ") nil t)
 	    (gnus-delete-line))
 	  (insert-buffer-substring nntp-server-buffer))))))
@@ -597,8 +598,9 @@ the actual number of articles toggled is returned."
   (save-excursion
     (set-buffer gnus-agent-current-history)
     (gnus-make-directory (file-name-directory gnus-agent-file-name))
-    (write-region (1+ (point-min)) (point-max)
-		  gnus-agent-file-name nil 'silent)))
+    (let ((coding-system-for-write gnus-agent-file-coding-system))
+      (write-region (1+ (point-min)) (point-max)
+		    gnus-agent-file-name nil 'silent))))
 
 (defun gnus-agent-close-history ()
   (when (gnus-buffer-live-p gnus-agent-current-history)
@@ -696,7 +698,7 @@ the actual number of articles toggled is returned."
 		(setq id "No-Message-ID-in-article")
 	      (setq id (buffer-substring (match-beginning 1) (match-end 1))))
 	    (let ((coding-system-for-write
-		   gnus-agent-article-file-coding-system))
+		   gnus-agent-file-coding-system))
 	      (write-region (point-min) (point-max)
 			    (concat dir (number-to-string (caar pos)))
 			    nil 'silent))
@@ -739,10 +741,12 @@ the actual number of articles toggled is returned."
   (save-excursion
     (while gnus-agent-buffer-alist
       (set-buffer (cdar gnus-agent-buffer-alist))
-      (write-region (point-min) (point-max)
-		    (gnus-agent-article-name ".overview"
-					     (caar gnus-agent-buffer-alist))
-		     nil 'silent)
+      (let ((coding-system-for-write
+	     gnus-agent-file-coding-system))
+	(write-region (point-min) (point-max)
+		      (gnus-agent-article-name ".overview"
+					       (caar gnus-agent-buffer-alist))
+		      nil 'silent))
       (pop gnus-agent-buffer-alist))
     (while gnus-agent-group-alist
       (nnheader-temp-write (caar gnus-agent-group-alist)
@@ -773,7 +777,9 @@ the actual number of articles toggled is returned."
 	    (gnus-agent-braid-nov group articles file))
 	  (gnus-make-directory (nnheader-translate-file-chars
 				(file-name-directory file)))
-	  (write-region (point-min) (point-max) file nil 'silent)
+ 	  (let ((coding-system-for-write
+ 		 gnus-agent-file-coding-system))
+ 	    (write-region (point-min) (point-max) file nil 'silent))
 	  (gnus-agent-save-alist group articles nil)
 	  (gnus-agent-enter-history
 	   "last-header-fetched-for-session"
@@ -1343,7 +1349,9 @@ The following commands are available:
 		 ;; Schedule the history line for nuking.
 		 (push (cdr elem) histories)))
 	     (gnus-make-directory (file-name-directory nov-file))
-	     (write-region (point-min) (point-max) nov-file nil 'silent)
+	     (let ((coding-system-for-write
+		    gnus-agent-file-coding-system))
+	       (write-region (point-min) (point-max) nov-file nil 'silent))
 	     ;; Delete the unwanted entries in the alist.
 	     (setq gnus-agent-article-alist
 		   (sort gnus-agent-article-alist 'car-less-than-car))
@@ -1368,7 +1376,7 @@ The following commands are available:
 	       (gnus-agent-save-alist group)
                ;; Mark all articles up to the first article
 	       ;; in `gnus-article-alist' as read.
-	       (when (caar gnus-agent-article-alist)
+	       (when (and info (caar gnus-agent-article-alist))
 		 (setcar (nthcdr 2 info)
 			 (gnus-range-add
 			  (nth 2 info)
@@ -1376,12 +1384,14 @@ The following commands are available:
 	       ;; Maybe everything has been expired from `gnus-article-alist'
 	       ;; and so the above marking as read could not be conducted,
 	       ;; or there are expired article within the range of the alist.
-	       (when (or (not (caar gnus-agent-article-alist))
-			 (> (car expired) (caar gnus-agent-article-alist)))  
-	       (setcar (nthcdr 2 info)
-		       (gnus-add-to-range
-			(nth 2 info)
-			(nreverse expired))))
+	       (when (and (car expired)
+			  (or (not (caar gnus-agent-article-alist))
+			      (> (car expired)
+				 (caar gnus-agent-article-alist))) )
+		 (setcar (nthcdr 2 info)
+			 (gnus-add-to-range
+			  (nth 2 info)
+			  (nreverse expired))))
 	       (gnus-dribble-enter
 		(concat "(gnus-group-set-info '"
 			(gnus-prin1-to-string info)
